@@ -1,6 +1,6 @@
 # ============================================================
 #
-# Copyright (C) 2014 Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
+# Copyright (C) 2014, 2015 Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
 #
 # This file may be licensed under the terms of the
 # GNU Lesser General Public License Version 3 (the ``LGPL''),
@@ -518,11 +518,15 @@ class IntrospectionSender (object):
 
         self.__server = rsb.createServer(processScope(self.__host.id or self.__host.hostname,
                                                       str(self.__process.id)))
-        def echo(event):
-            metaData = event.metaData
-            metaData.setUserTime('request.send',    metaData.sendTime)
-            metaData.setUserTime('request.receive', metaData.receiveTime)
-            return event
+        def echo(request):
+            reply = rsb.Event(scope = request.scope,
+                              data = request.data,
+                              type = type(request.data))
+            reply.metaData.setUserTime('request.send',
+                                       request.metaData.sendTime)
+            reply.metaData.setUserTime('request.receive',
+                                       request.metaData.receiveTime)
+            return reply
         self.__server.addMethod('echo', echo,
                                 requestType = rsb.Event,
                                 replyType   = rsb.Event)
